#ifndef GC_H_SEEN
#define GC_H_SEEN

void gc_init(void *stackbase);  // call with high stack addr, e.g. argv

void *new(size_t size);
void *dup(void *p, size_t size);

void gc(void);
void grow(void);

void gc_stats(FILE *f);

#define NEW(TYPE)            ((TYPE *)new(sizeof(TYPE)))
#define NEWA(TYPE, NUM)      ((TYPE *)new((NUM)*sizeof(TYPE)))
#define DUP(PTR, TYPE)       ((TYPE *)dup(PTR, sizeof(TYPE)))
#define DUPA(PTR, TYPE, NUM) ((TYPE *)dup(PTR, (NUM)*sizeof(TYPE)))


// externals

typedef struct external external_t;
struct external {
    external_t *next;
    void *addrmin;  // lowest possible address
    void *addrmax;  // highest possible address + 1
    int (*scanref)(void (*scan)(void *p, size_t n, uint8_t sense),
                   void (*scanref)(void *x, uint8_t sense),
                   external_t *ext, void *x, uint8_t sense);
        // is 'x' a valid reference to an object in 'ext'?
        // is it already marked with 'sense'? if not, mark and scan.
        // return 1 if 'x' referenced an object in 'ext', 0 otherwise.
    void (*sweep)(external_t *ext);
        // free all objects marked with 0.
    void (*stats)(external_t *ext, FILE *f);
        // print statistics to f; ignored if NULL.
};

void gc_add_external(external_t *ext);

#endif // GC_H_SEEN
